@@ -1,4 +1,7 @@
-from visidata import *
+import os
+import os.path
+
+from visidata import VisiData, vd, Path, BaseSheet, TableSheet, TextSheet, SettableColumn
 
 
 vd.option('filetype', '', 'specify file type', replay=True)
@@ -6,7 +9,13 @@ vd.option('filetype', '', 'specify file type', replay=True)
 
 @VisiData.api
 def inputFilename(vd, prompt, *args, **kwargs):
-    return vd.input(prompt, type="filename", *args, completer=_completeFilename, **kwargs).strip()
+    completer= _completeFilename
+    if not vd.couldOverwrite():  #1805 don't suggest an existing file
+        completer = None
+        v = kwargs.get('value', '')
+        if v and Path(v).exists():
+            kwargs['value'] = ''
+    return vd.input(prompt, type="filename", *args, completer=completer, **kwargs).strip()
 
 
 @VisiData.api
@@ -35,6 +44,38 @@ def _completeFilename(val, state):
     return files[state%len(files)]
 
 
+@VisiData.api
+def guessFiletype(vd, p, *args, funcprefix='guess_'):
+    '''Call all vd.guess_<filetype>(p) functions and return best candidate sheet based on file contents.'''
+
+    guessfuncs = [getattr(vd, x) for x in dir(vd) if x.startswith(funcprefix)]
+    filetypes = []
+    for f in guessfuncs:
+        try:
+            filetype = f(p, *args)
+            if filetype:
+                filetype['_guesser'] = f.__name__
+                filetypes.append(filetype)
+        except FileNotFoundError:
+            pass
+        except Exception as e:
+            vd.debug(f'{f.__name__}: {e}')
+
+    if filetypes:
+        return sorted(filetypes, key=lambda r: -r.get('_likelihood', 1))[0]
+
+    return {}
+
+
+@VisiData.api
+def guess_extension(vd, path):
+    # try auto-detect from extension
+    ext = path.suffix[1:].lower()
+    openfunc = getattr(vd, f'open_{ext}', vd.getGlobals().get(f'open_{ext}'))
+    if openfunc:
+        return dict(filetype=ext, _likelihood=3)
+
+
 @VisiData.api
 def openPath(vd, p, filetype=None, create=False):
     '''Call ``open_<filetype>(p)`` or ``openurl_<p.scheme>(p, filetype)``.  Return constructed but unloaded sheet of appropriate type.
@@ -53,27 +94,43 @@ def openPath(vd, p, filetype=None, create=False):
         return None
 
     if not filetype:
-        if p.is_dir():
-            filetype = 'dir'
-        else:
-            filetype = p.ext or options.filetype or 'txt'
+        filetype = p.ext or vd.options.filetype
 
     filetype = filetype.lower()
 
     if not p.exists():
-        if not create:
-            return None
         newfunc = getattr(vd, 'new_' + filetype, vd.getGlobals().get('new_' + filetype))
         if not newfunc:
             vd.warning('%s does not exist, creating new sheet' % p)
-            return vd.newSheet(p.name, 1, source=p)
+            return vd.newSheet(p.base_stem, 1, source=p)
 
         vd.status('creating blank %s' % (p.given))
         return newfunc(p)
 
-    openfunc = getattr(vd, 'open_' + filetype, vd.getGlobals().get('open_' + filetype))
+    if p.is_fifo():
+        # read the file as text, into a RepeatFile that can be opened multiple times
+        p = Path(p.given, fp=p.open(mode='rb'))
+
+    openfuncname = 'open_' + filetype
+    openfunc = getattr(vd, openfuncname, vd.getGlobals().get(openfuncname))
     if not openfunc:
+        opts = vd.guessFiletype(p)
+        if opts and 'filetype' in opts:
+            filetype = opts['filetype']
+            openfuncname = 'open_' + filetype
+            openfunc = getattr(vd, openfuncname, vd.getGlobals().get(openfuncname))
+            if not openfunc:
+                vd.error(f'guessed {filetype} but no {openfuncname}')
+
+            vs = openfunc(p)
+            for k, v in opts.items():
+                if k != 'filetype' and not k.startswith('_'):
+                    setattr(vs.options, k, v)
+            vd.warning('guessed "%s" filetype based on contents' % opts['filetype'])
+            return vs
+
         vd.warning('unknown "%s" filetype' % filetype)
+
         filetype = 'txt'
         openfunc = vd.open_txt
 
@@ -81,13 +138,16 @@ def openPath(vd, p, filetype=None, create=False):
 
     return openfunc(p)
 
-
 @VisiData.api
 def openSource(vd, p, filetype=None, create=False, **kwargs):
     '''Return unloaded sheet object for *p* opened as the given *filetype* and with *kwargs* as option overrides. *p* can be a Path or a string (filename, url, or "-" for stdin).
     when true, *create* will return a blank sheet, if file does not exist.'''
-    if not filetype:
-        filetype = options.getonly('filetype', 'global', '')
+
+    if isinstance(p, BaseSheet):
+        return p
+
+    filetype = filetype or vd.options.getonly('filetype', str(p), '')  #1710
+    filetype = filetype or vd.options.getonly('filetype', 'global', '')
 
     vs = None
     if isinstance(p, str):
@@ -111,26 +171,21 @@ def openSource(vd, p, filetype=None, create=False, **kwargs):
 def open_txt(vd, p):
     'Create sheet from `.txt` file at Path `p`, checking whether it is TSV.'
     if p.exists(): #1611
-        with p.open_text(encoding=vd.options.encoding) as fp:
+        with p.open(encoding=vd.options.encoding) as fp:
+            delimiter = vd.options.delimiter
             try:
-                if options.delimiter in next(fp):    # peek at the first line
+                if delimiter and delimiter in next(fp):    # peek at the first line
                     return vd.open_tsv(p)  # TSV often have .txt extension
             except StopIteration:
-                return Sheet(p.name, columns=[SettableColumn()], source=p)
-    return TextSheet(p.name, source=p)
-
-
-@VisiData.api
-def loadInternalSheet(vd, cls, p, **kwargs):
-    'Load internal sheet of given class.'
-    vs = cls(p.name, source=p, **kwargs)
-    options._set('encoding', 'utf8', vs)
-    if p.exists():
-#        vd.sheets.insert(0, vs) # broke replay with macros.reload()
-        vs.reload.__wrapped__(vs)
-#        vd.sheets.pop(0)
-    return vs
+                return TableSheet(p.base_stem, columns=[SettableColumn()], source=p)
+    return TextSheet(p.base_stem, source=p)
 
 
 BaseSheet.addCommand('o', 'open-file', 'vd.push(openSource(inputFilename("open: "), create=True))', 'Open file or URL')
 TableSheet.addCommand('zo', 'open-cell-file', 'vd.push(openSource(cursorDisplay) or fail(f"file {cursorDisplay} does not exist"))', 'Open file or URL from path in current cell')
+BaseSheet.addCommand('gU', 'undo-last-quit', 'push(allSheets[-1])', 'reopen most recently closed sheet')
+
+vd.addMenuItems('''
+    File > Open > input file/url > open-file
+    File > Reopen last closed > undo-last-quit
+''')
