@@ -1,20 +1,20 @@
-from visidata import VisiData, vd, Sheet, date, anytype, options, Column, Progress, ColumnItem, vlen, PyobjSheet, TypedWrapper, InferColumnsSheet
+from visidata import VisiData, vd, Sheet, date, anytype, options, Column, Progress, ColumnItem, vlen, PyobjSheet, TypedWrapper
 
 'Loaders for .npy and .npz.  Save to .npy.  Depends on the zip loader.'
 
 @VisiData.api
 def open_npy(vd, p):
-    return NpySheet(p.name, source=p)
+    return NpySheet(p.base_stem, source=p)
 
 @VisiData.api
 def open_npz(vd, p):
-    return NpzSheet(p.name, source=p)
+    return NpzSheet(p.base_stem, source=p)
 
 vd.option('npy_allow_pickle', False, 'numpy allow unpickling objects (unsafe)')
 
 class NpySheet(Sheet):
     def iterload(self):
-        import numpy
+        numpy = vd.importExternal('numpy')
         if not hasattr(self, 'npy'):
             self.npy = numpy.load(str(self.source), encoding='bytes', **self.options.getall('npy_'))
         self.reloadCols()
@@ -47,12 +47,12 @@ class NpzSheet(vd.ZipSheet):
     ]
 
     def iterload(self):
-        import numpy
+        numpy = vd.importExternal('numpy')
         self.npz = numpy.load(str(self.source), encoding='bytes', **self.options.getall('npy_'))
         yield from Progress(self.npz.items())
 
     def openRow(self, row):
-        import numpy
+        numpy = vd.importExternal('numpy')
         tablename, tbl = row
         if isinstance(tbl, numpy.ndarray):
             return NpySheet(tablename, npy=tbl)
@@ -62,7 +62,7 @@ class NpzSheet(vd.ZipSheet):
 
 @VisiData.api
 def save_npy(vd, p, sheet):
-    import numpy as np
+    np = vd.importExternal('numpy')
 
     dtype = []
 
