@@ -1,5 +1,4 @@
 from functools import wraps
-from unittest import mock
 import curses
 
 import visidata
@@ -25,13 +24,13 @@ def asyncthread(func):
 
 
 class VisiData(visidata.Extensible):
-    allPrefixes = ['g', 'z', ESC]  # embig'g'en, 'z'mallify, ESC=Alt/Meta
+    allPrefixes = ['g', 'z', 'Alt+']  # embig'g'en, 'z'mallify, Alt/Esc=User
 
     @classmethod
     def global_api(cls, func):
         'Make global func() and identical vd.func()'
         def _vdfunc(*args, **kwargs):
-            return func(visidata.vd, *args, **kwargs)
+            return getattr(visidata.vd, func.__name__)(*args, **kwargs)
         visidata.vd.addGlobals({func.__name__: func})
         setattr(cls, func.__name__, func)
         return wraps(func)(_vdfunc)
@@ -40,10 +39,18 @@ class VisiData(visidata.Extensible):
         self.sheets = []  # list of BaseSheet; all sheets on the sheet stack
         self.allSheets = []  # list of all non-precious sheets ever pushed
         self.lastErrors = []
+        self.pendingKeys = []
         self.keystrokes = ''
-        self.scrFull = mock.MagicMock(__bool__=mock.Mock(return_value=False))  # disable curses in batch mode
+        self.scrFull = None
         self._cmdlog = None
+        self.currentReplay = None
         self.contexts = [self]  # objects whose attributes are in the fallback context for eval/exec.
+        self.importingModule = None
+        self.importedModules = []
+
+    @property
+    def cursesEnabled(self):
+        return bool(self.scrFull)
 
     def sheetstack(self, pane=0):
         'Return list of sheets in given *pane*. pane=0 is the active pane.  pane=-1 is the inactive pane.'
@@ -73,10 +80,6 @@ class VisiData(visidata.Extensible):
     def activeStack(self):
         return self.sheetstack() or self.sheetstack(-1)
 
-    @visidata.drawcache_property
-    def mousereg(self):
-        return []
-
     def __copy__(self):
         'Dummy method for Extensible.init()'
         pass
@@ -99,18 +102,56 @@ class VisiData(visidata.Extensible):
         'Invalidate internal caches between command inputs.'
         visidata.Extensible.clear_all_caches()
 
+    def resetVisiData(self):
+        self.clearCaches()  # we want vd to return a new VisiData object for each command
+        vd = visidata.vd  # get the new vd
+        vd.cmdlog.rows = []
+        vd.sheets = []
+        vd.allSheets = []
+        return vd
+
+    def get_wch(self, scr):
+        try:
+            return scr.get_wch()
+        except AttributeError:  #192 some packages don't have wide chars
+            k = scr.getch()
+            if k == -1:  # mimic get_wch behavior
+                raise curses.error('no char ready')
+            return k
+
+    def drainPendingKeys(self, scr):
+        '''Call scr.get_wch() until no more keypresses are available.  Return True if any keypresses are pending.'''
+        scr.timeout(0)
+        try:
+            while True:
+                k = self.get_wch(scr)
+                if k:
+                    self.pendingKeys.append(k)
+                else:
+                    break
+        except curses.error:
+            pass
+        finally:
+            scr.timeout(self.curses_timeout)
+
+        return bool(self.pendingKeys)
+
     def getkeystroke(self, scr, vs=None):
         'Get keystroke and display it on status bar.'
+        self.drainPendingKeys(scr)
         k = None
-        curses.reset_prog_mode()  #1347
-        try:
-            scr.refresh()
-            k = scr.get_wch()
-            vs = vs or self.activeSheet
-            if vs:
-                self.drawRightStatus(vs._scr, vs) # continue to display progress %
-        except curses.error:
-            return ''  # curses timeout
+        if self.pendingKeys:
+            k = self.pendingKeys.pop(0)
+        else:
+            curses.reset_prog_mode()  #1347
+            try:
+                scr.refresh()
+                k = self.get_wch(scr)
+                vs = vs or self.activeSheet
+                if vs:
+                    self.drawRightStatus(vs._scr, vs) # continue to display progress %
+            except curses.error:
+                return ''  # curses timeout
 
         if isinstance(k, str):
             if ord(k) >= 32 and ord(k) != 127:  # 127 == DEL or ^?
@@ -118,14 +159,6 @@ class VisiData(visidata.Extensible):
             k = ord(k)
         return curses.keyname(k).decode('utf-8')
 
-    def onMouse(self, scr, y, x, h, w, **kwargs):
-        self.mousereg.append((scr, y, x, h, w, kwargs))
-
-    def getMouse(self, _scr, _x, _y, button):
-        for scr, y, x, h, w, kwargs in self.mousereg[::-1]:
-            if scr is _scr and x <= _x < x+w and y <= _y < y+h and button in kwargs:
-                return kwargs[button]
-
     @property
     def screenHeight(self):
         return self.scrFull.getmaxyx()[0] if self.scrFull else 25
